/**
 * MPQAAThread.java: MPQ Audio Analyzer.
 *            Copyright (C) 2013 donfede@casagrau.org
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * created 2013-01-27 donfede@casagrau.org
 * $Id:#
 */

package org.casagrau.mpq.audio_service;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

import android.content.Context;
import android.media.AudioFormat;
import android.os.Environment;
import android.os.Handler;
import android.util.Log;

public class MPQAAThread extends MPQAudioWorkerThread {
	private static final String LOGTAG = "MPQAAThread";

	
	/** 
	 * Constructor for the worker thread.
	 * {@link #MPQAudioWorkerThread}  
	 */
	public MPQAAThread(Handler parentHandler, String filename, long filepos, int state, Context context) {
		super(parentHandler, filename, filepos, state, context);
	}
	

	/**
	 * Perform basic audio analysis.
	 * This does not return data, helper methods should be used for that (ex: getLen() ).
	 */
	public void run() {
		Log.d(LOGTAG, "runAA() 00 filename: " + mFilename + " len: " + mFilelen + " noise " + mFilenoise);
		// FIXME: to implement later, when using threads -- trigger handler method to update meta-data and notify listeners. 

// check for reusable file with cached data
		
//		mFilelen = 0; // FIXME: this may be deleted possibly???
//		mFilenoise = mFilelen; // FIXME: this may be deleted possibly???
		
		
		
		/////////////////////
		
		/**
		 * Open file.
		 */
		// FIXME: this is a block of repeated code from MPQAudioWorkerPlayThread -- think how to better organize things...
		// Get the file we want to playback.
		File lFile = new File(Environment.getExternalStorageDirectory().getAbsolutePath() + mFilename);
		Log.d(LOGTAG, "runAA() 06  file:" + lFile);
		// Open file to play back
		try {
			mFileHandle = new RandomAccessFile(lFile, "r");
		} catch (FileNotFoundException e) {
			Log.e(LOGTAG,"runAA() Open failed e:" + e.getLocalizedMessage());
			e.printStackTrace();
		}

		// FIXME: read WAVE file headers
		int lfrequency = 11025; 
		int lchannelConfiguration = AudioFormat.CHANNEL_OUT_MONO;
		int laudioEncoding = AudioFormat.ENCODING_PCM_16BIT; 
		
    	short lchannelCount;
    	if (AudioFormat.CHANNEL_OUT_MONO == lchannelConfiguration) lchannelCount = 1;
    	else if (AudioFormat.CHANNEL_OUT_STEREO == lchannelConfiguration) lchannelCount = 2;
    	else throw new IllegalStateException("unsuported channel configuration: " + lchannelConfiguration);

    	short lsampleBit = 0; // sample bitrate
    	if (AudioFormat.ENCODING_PCM_16BIT == laudioEncoding) lsampleBit = 16;
    	else if (AudioFormat.ENCODING_PCM_8BIT == laudioEncoding) lsampleBit = 8;
    	else throw new IllegalStateException("unsuported sample bitrate: " + lsampleBit);

    	int lframeSize = (lsampleBit/8)*lchannelCount;
    	Log.d(LOGTAG, "runAA() 20  framesize:" + lframeSize + " channelCount: " + lchannelCount + " sampleBit: " + lsampleBit);
		
    	
		// Calculate audio length // FIXME: currently this is a BAD guess.  This needs to read the headers. 
    	long lfilebytes;
		try {
			lfilebytes = mFileHandle.length();
		} catch (IOException e) {
			Log.e(LOGTAG,"runAA() file length failed e:" + e.getLocalizedMessage());
			e.printStackTrace();
			return;
		}
    	Log.d(LOGTAG, "runAA() 22  filebytes:" + lfilebytes + " fileb - head: " + (lfilebytes - 40) );
    	lfilebytes = lfilebytes - 40; // FIXME: crappy ballpark guess at headers -- this should be replaced with actually reading wave headers and subtracting from size OR get WAVE data chunk size
    	long lframeCount = lfilebytes / lframeSize;
		double lfrequencyMS = lfrequency / 1000.0; // cycles per millisecond
    	long lfileMS = (long) (lframeCount / lfrequencyMS); // FIXME: review/re-think if this float division is necessary...
    	Log.d(LOGTAG, "runAA() 24  fileMS: " + lfileMS + " frameCount: " + lframeCount + " lfreq: " + lfrequency + " lfreqMS: " + lfrequencyMS);
    	

		// Calculate noise length // FIXME: currently this is just BAD.  This needs to actually read the file. 
    	long lnoiseMS;
lnoiseMS = (long) ((float)lfileMS * 0.75) ; // FIXME: evil fill in hack before actually calculating noise. 
Log.d(LOGTAG, "runAA() 24  noiseMS: " + lnoiseMS);
    	
//		// move cursor past wave header (dumb for now; actually read the header later)
//		mFileHandle.seek(40);
//		// Create a buffer for audio playback that is appropriate for input file.
//		int lbufferSize = 16384;
//		byte[] buffer = new byte[lbufferSize];
//		
//		int lbufferReadSize;	// Variable to store the size read into the buffer.
//		int lplaySize;			// Variable to store the size played to the speakers/AudioTrack.
//		mFilepos = 0;
//		long lFileposBytes = 0;
//
//		// Audio Analysis loop -- // FIXME: review what waveformview does, and redo this integrating that
//		while ( true ) {
//			Log.d(LOGTAG, "runAA() 70  IN loop read -  lFileposBytes:" + lFileposBytes + " mFilepos: " + mFilepos);			
//
//				lbufferReadSize = mFileHandle.read(buffer);
//
//				Log.d(LOGTAG, "runAA() 73A        IN loop  PLAY read: " + lbufferReadSize + " buf says:" + buffer.length + " mStateThread: " + mStateThread);
//				Log.d(LOGTAG, "runAA() 73B        IN loop  PLAY filepos: " + mFileHandle.getFilePointer() + " file len: " + mFileHandle.length() );
//
//				if (lbufferReadSize > 0) {
//					lFileposBytes = mFileHandle.getFilePointer() - 40; // FIXME: crummy approximation of audio length, without headers.  this should be read from actual headers...
//					mFilepos = cursorFrameToMS(lFileposBytes, lframeSize, lfrequency);
//					Log.d(LOGTAG, "runAA() 75 wrote filepos: " + mFilepos + " filepos STR: " + SessionsDB.longSessionMSecString(mFilepos));
//				} else 
//					break; // We've likely hit EOF.  // FIXME: should probably actually check for EOF... (but we're stopping either way...)
//		} // END while() playback loop
//		Log.d(LOGTAG, "runAA() 90 OUT AA loop" );


		// Send updates.  Report results back (from child thread to parent service; from service to SessionDetailFragment to DB).
		sendPMsgAA(lfileMS, lnoiseMS);

// update image/data for Waveformview and save it to a reusable file // FIXME: implement 

		
		
// // FIXME: do we need member attributes? this goes away asap... --		Log.d(LOGTAG, "run() 99 filename: " + mFilename + " len: " + mFilelen + " noise " + mFilenoise);
		Log.d(LOGTAG, "run() 99 filename: " + mFilename + " len: " + lfileMS + " noise " + lnoiseMS);
		return;
	}
	
//	/**
//	 * Returns file total audio length, in milliseconds.
//	 * @return long
//	 */
//	public long getLen() {
//		Log.d(LOGTAG, "getLen() 00 filename: " + mFilename + " len: " + mFilelen);
//		return mFilelen;
//	}
//	
//	/**
//	 * Returns file noise length, in milliseconds.
//	 * @return long
//	 */
//	public long getNoise() {
//		Log.d(LOGTAG, "getNoise() 00 filename: " + mFilename + " noise: " + mFilenoise);
//		return mFilenoise;
//	}
	
	// FIXME: should return link or status of cache (of image or audio data analysis) -- to be used by Waveformview control.
	
} // END class MPQAAThread
