/**
 * MPQAudioWorkerPlayThread.java: Playback worker thread class.
 *            Copyright (C) 2012 donfede@casagrau.org
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * created 2012-12-04 donfede@casagrau.org
 * credit to: Reto Meier, EMEA Android Developer Advocate, Google UK; for a record and playback example -- http://eurodev.blogspot.com/2009/09/raw-audio-manipulation-in-android.html
 *			  Stjepan Rajko, urbanSTEW -- (GPL) RehearsalAssistant/RehearsalAudioRecorder.java
 *			  cristi (Cristian Baita) -- http://indyvision.net/2010/02/android-threads-tutorial-part-3/
 * $Id: MPQAudioWorkerPlayThread.java,v 1.2 2013-01-28 01:58:06 donfede Exp $
 */

package org.casagrau.mpq.audio_service;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.casagrau.mpq.mpq_android.SessionsDBProvider.SessionsDB;

import android.content.Context;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioTrack;
import android.os.Environment;
import android.os.Handler;
import android.util.Log;



public class MPQAudioWorkerPlayThread extends MPQAudioWorkerThread {
	private static final String LOGTAG = "MPQAudioWorkerPlayThread";

	/** 
	 * Constructor for the worker thread.
	 * {@link #MPQAudioWorkerThread}  
	 */
	public MPQAudioWorkerPlayThread(Handler parentHandler, String filename, long filepos, int state, Context context) {
		super(parentHandler, filename, filepos, state, context);
	}
	
	/**
	 * Play audio from a wave file
	 * Credit to: Reto Meier, EMEA Android Developer Advocate, Google UK -- http://eurodev.blogspot.com/2009/09/raw-audio-manipulation-in-android.html
	 * @param intent with the action and extra content to process. 
	 * 					Extra ARG_FILENAME is checked for to open.
	 * @throws IllegalStateException if there are errors writing the file
	 */
	//FIXME: make private
	public void run() { 
		Log.d(LOGTAG, "runPLAY() 00");
		// Get the file we want to playback.
		File lFile = new File(Environment.getExternalStorageDirectory().getAbsolutePath() + mFilename);
		Log.d(LOGTAG, "runPLAY() 06  file:" + lFile + "mStateThread: " + mStateThread);

		try {
			// Open file to play back
			mFileHandle = new RandomAccessFile(lFile, "r");

			// FIXME: read WAVE file headers
			int lfrequency = 11025; 
			int lchannelConfiguration = AudioFormat.CHANNEL_OUT_MONO;
			int laudioEncoding = AudioFormat.ENCODING_PCM_16BIT; 
			
	    	short lchannelCount;
	    	if (AudioFormat.CHANNEL_OUT_MONO == lchannelConfiguration) lchannelCount = 1;
	    	else if (AudioFormat.CHANNEL_OUT_STEREO == lchannelConfiguration) lchannelCount = 2;
	    	else throw new IllegalStateException("unsuported channel configuration: " + lchannelConfiguration);

	    	short lsampleBit = 0; // sample bitrate
	    	if (AudioFormat.ENCODING_PCM_16BIT == laudioEncoding) lsampleBit = 16;
	    	else if (AudioFormat.ENCODING_PCM_8BIT == laudioEncoding) lsampleBit = 8;
	    	else throw new IllegalStateException("unsuported sample bitrate: " + lsampleBit);

	    	int lframeSize = (lsampleBit/8)*lchannelCount;
	    	Log.d(LOGTAG, "runPLAY() 20  framesize:" + lframeSize + " channelCount: " + lchannelCount + " sampleBit: " + lsampleBit);
			
			// Calculate and report audio length // FIXME: currently this is a BAD guess.  This needs to read the headers. 
	    	long lfilebytes = mFileHandle.length();
	    	Log.d(LOGTAG, "runPLAY() 22  filebytes:" + lfilebytes + " fileb - head: " + (lfilebytes - 40) );
	    	lfilebytes = lfilebytes - 40; // FIXME: crappy ballpark guess at headers -- this should be replaced with actually reading wave headers and subtracting from size OR get WAVE data chunk size
	    	long lframeCount = lfilebytes / lframeSize;
			double lfrequencyMS = lfrequency / 1000.0; // cycles per millisecond
	    	long lfileMS = (long) (lframeCount / lfrequencyMS); // FIXME: review/re-think if this float division is necessary...
	    	Log.d(LOGTAG, "runPLAY() 24  fileMS: " + lfileMS + " frameCount: " + lframeCount + " lfreq: " + lfrequency + " lfreqMS: " + lfrequencyMS);
			sendPMsgLen(lfileMS);

			// move cursor past wave header (dumb for now; actually read the header later)
			mFileHandle.seek(40);

			// Create a buffer for audio playback that is appropriate for input file.
			int lbufferSize = AudioTrack.getMinBufferSize(lfrequency, lchannelConfiguration, laudioEncoding);
			lbufferSize = 2 * lbufferSize;
			if (lbufferSize < 16384) lbufferSize = 16384; // FIXME: hack to increase buffer size (saw it at 2400) -- better solution would be to 'do math' and try and buffer $X milliseconds
			byte[] buffer = new byte[lbufferSize];
			Log.d(LOGTAG, "runPLAY() 30  bufSize(2xmin, or 16384): " + lbufferSize);
			// Create a new AudioTrack object using the same parameters as the AudioRecord
			// object used to create the file.
			AudioTrack laudioTrack = new AudioTrack(AudioManager.STREAM_MUSIC,
					lfrequency,
					lchannelConfiguration,
					laudioEncoding,
					lbufferSize,
					AudioTrack.MODE_STREAM);

			if (laudioTrack.getState() != AudioTrack.STATE_INITIALIZED)
				throw new IOException("runPLAY() AudioTrack initialization failed");

			// Play up to $X seconds (or pause for $Y seconds) FIXME: make this a Preferences setting
			laudioTrack.play();
			long lnowTime = System.currentTimeMillis();
			SimpleDateFormat formatter = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
			String lstartTimeStr = formatter.format(new Date(lnowTime));
			Log.d(LOGTAG, "runPLAY() 50  start time" + lnowTime + "str start: " + lstartTimeStr);
			Log.d(LOGTAG, "runPLAY() 52 playbackNotificationRate:" + laudioTrack.getPositionNotificationPeriod() );
			
			int lbufferReadSize;	// Variable to store the size read into the buffer.
			int lplaySize;			// Variable to store the size played to the speakers/AudioTrack.
			mFilepos = 0;
			long lFileposBytes = 0;
			sendPMsgPos(mFilepos, false);

			// Playback loop
			while ( (MPQAudioService.FLG_SRV_STATE_PLAY == mStateThread) || 
					(MPQAudioService.FLG_SRV_STATE_PAUSE_PLAY == mStateThread) ) {
				Log.d(LOGTAG, "runPLAY() 70  IN loop read -  headPos:" + laudioTrack.getPlaybackHeadPosition() + " mStateThread: " + mStateThread);
				Log.d(LOGTAG, "runPLAY() 72             now time:" + formatter.format(new Date(lnowTime)) );
				
				if (MPQAudioService.FLG_SRV_STATE_PLAY == mStateThread) {
					if (AudioTrack.PLAYSTATE_PLAYING != laudioTrack.getPlayState()  ) 
						laudioTrack.play();
					lbufferReadSize = mFileHandle.read(buffer);
					Log.d(LOGTAG, "runPLAY() 73A        IN loop  PLAY read: " + lbufferReadSize + " buf says:" + buffer.length + " headPos:" + laudioTrack.getPlaybackHeadPosition() + " mStateThread: " + mStateThread);
					Log.d(LOGTAG, "runPLAY() 73B        IN loop  PLAY filepos: " + mFileHandle.getFilePointer() + " file len: " + mFileHandle.length() );

					if (lbufferReadSize > 0) {
						// Write the music buffer to the AudioTrack object
						// FIXME: i fear i should have a ring buffer here (and read, separately from the audio playback...)
						lplaySize = laudioTrack.write(buffer, 0, lbufferReadSize);
						lFileposBytes = mFileHandle.getFilePointer() - 40; // FIXME: crummy approximation of audio length, without headers.  this should be read from actual headers...
						mFilepos = cursorFrameToMS(lFileposBytes, lframeSize, lfrequency);
						sendPMsgPos(mFilepos, false);
						Log.d(LOGTAG, "runPLAY() 74 wrote playSize: " + lplaySize + " headPos:" + laudioTrack.getPlaybackHeadPosition() + " filepos: " + mFilepos );
						Log.d(LOGTAG, "runPLAY() 75 wrote filepos: " + mFilepos + " filepos STR: " + SessionsDB.longSessionMSecString(mFilepos));
					} else 
						break; // We've likely hit EOF.  // FIXME: should probably actually check for EOF... (but we're stopping either way...)
				} else { // FLG_SRV_STATE_PAUSE_PLAY
					if (AudioTrack.PLAYSTATE_PAUSED != laudioTrack.getPlayState()  ) 
						laudioTrack.pause();
					// wait before checking again.
					try {
						Thread.sleep(AUDIO_PAUSE_MS);
					} catch (Exception e) {
						e.getLocalizedMessage();
					}	
					Log.d(LOGTAG, "recordWAV() 77      IN while PAUSE WAIT mStateThread: " + mStateThread + " nowTime:" + formatter.format(new Date(lnowTime)) );            		
				} 	
				lnowTime = System.currentTimeMillis();
			} // END while() playback loop
			Log.d(LOGTAG, "runPLAY() 90 OUT playing while now/end time:" + formatter.format(new Date(lnowTime)) );
			laudioTrack.stop();
			laudioTrack.release();

			// Set state to stopped, and send updates.
			mFilepos = 0;
			sendPMsgPos(mFilepos, true);
			if (MPQAudioService.FLG_SRV_STATE_STOP != mStateThread) {		
	    		Log.d(LOGTAG, "runPLAY() 94 send startService intent STOP");
	    		sendSvcIntentStop();
			}
			// FIXME: code below is exact duplicate of MPQAudioWorkerRecThread() -- should probably be moved into method in parent class...
    		Log.d(LOGTAG, "runPLAY() 97 send startService intent OPEN");
    		sendSvcIntentOpen();

		} catch (Exception e) {
			Log.e(LOGTAG,"runPLAY() Playback Failed e:" + e.getLocalizedMessage());
		}
		Log.d(LOGTAG, "runPLAY() 99");
	} // END runPLAY()


} // END class MPQAudioWorkerPlayThread





