/**
 * SessionsSummaryFragment.java: Welcome screen with sessions summary histogram.
 *            Copyright (C) 2012,2013 donfede@casagrau.org
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * created 2012-09-23 donfede@casagrau.org
 * credit to: AChartEngine contact [at] achartengine.org for an excellent graphing library - http://www.achartengine.org/content/goodies.html
 *
 * $Id: SessionsSummaryFragment.java,v 1.12 2013-01-25 01:01:40 donfede Exp $
 */
package org.casagrau.mpq.mpq_android;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.achartengine.ChartFactory;
import org.achartengine.GraphicalView;
import org.achartengine.chart.BarChart.Type;
import org.achartengine.chart.PointStyle;
import org.achartengine.model.CategorySeries;
import org.achartengine.model.MultipleCategorySeries;
import org.achartengine.model.TimeSeries;
import org.achartengine.model.XYMultipleSeriesDataset;
import org.achartengine.model.XYSeries;
import org.achartengine.renderer.DefaultRenderer;
import org.achartengine.renderer.SimpleSeriesRenderer;
import org.achartengine.renderer.XYMultipleSeriesRenderer;
import org.achartengine.renderer.XYSeriesRenderer;
import org.casagrau.mpq.mpq_android.SessionsDBProvider.SessionsDB;

import android.database.Cursor;
import android.graphics.Color;
import android.graphics.Paint.Align;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.CursorLoader;
import android.support.v4.content.Loader;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.LinearLayout.LayoutParams;

/**
 * Fragment with GUI for Sessions Summary.
 */
public class SessionsSummaryFragment extends Fragment implements 
		LoaderManager.LoaderCallbacks<Cursor> {
	private static final String LOGTAG = "SessionsSummaryFragment";	
	
	// Unique integer identifiers for Loaders; FIXME: rather warty (repeated in DetailFragment, and SessionsListFragment)... :(
	private static final int LDR_SESSIONS = 2;

	private GraphicalView mChartView;	// chart container/view
	private XYMultipleSeriesDataset mChartDataset;  // data container for the chart
	private XYMultipleSeriesRenderer mChartRenderer;// renderer; used to update axes
	private XYSeries mChartSeriesTotal;				// series of the total length of time
	private XYSeries mChartSeriesNoise;				// series of the noise subset length of time

	// Parameterless constructor, required by framework
	public SessionsSummaryFragment() {
		super();
	}
	
	/**
	 * Create the View for this fragment - bringing in XML layout with summary histogram and button to start practice.
	 */
	@Override	
	public View onCreateView(LayoutInflater inflater, ViewGroup container, 
			Bundle savedInstanceState) {
		View v = inflater.inflate(R.layout.fragment_sessionssummary, container, false);
		return v;
	}
	
	@Override
	public void onResume() {
		super.onResume();
        Log.d(LOGTAG, "DEBUG 00 onResume()");
        
        if (null == mChartDataset) {
        	mChartDataset = new XYMultipleSeriesDataset();
        }

		if (null == mChartView) {
	        Log.d(LOGTAG, "DEBUG 21 onResume()");
			
			mChartView = makeChartView();
			
			LinearLayout layout = (LinearLayout) getActivity().findViewById(R.id.recent_practice_chart);
			layout.addView(mChartView, new LayoutParams (LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT) );
			Log.d(LOGTAG, "DEBUG 29 onResume()");
		}

		// FIXME: no luck with "repainting" bug, where chart screen ends up white.
		//mChartView.repaint();
		//mChartView.invalidate();
		

		// Prepare the Loaders, triggering callback methods.
		getLoaderManager().initLoader(LDR_SESSIONS, null, this);
		
		Log.d(LOGTAG, "DEBUG 99 onResume()");
		return;
	}

	
    private GraphicalView makeChartView() {
        Log.d(LOGTAG, "DEBUG 00 makeChartView()");
        
        GraphicalView v;
        
        // Dummy/empty data for the mock graph report (to be later replaced by onLoadFinished() Loader).
        String[] titles = new String[] { "Total", "Noise" };
        List<double[]> values = new ArrayList<double[]>();
        
        values.add(new double[] { });
      	values.add(new double[] { });
        int[] colors = new int[] { Color.BLUE, Color.YELLOW };
	  
		mChartDataset = buildBarDataset(titles, values);
		// Get handles to data series, to manipulate them later. 
		mChartSeriesTotal = mChartDataset.getSeriesAt(0); 
		mChartSeriesNoise = mChartDataset.getSeriesAt(1);
        
        
        // Create renderer and prepare chart titles
        mChartRenderer = buildBarRenderer(colors);
 // FIXME: set axis values
        setChartSettings(mChartRenderer, "Recent Practice Sessions", "Date", "Length (minutes)", 0.5, 12.5, 0, 130, Color.GRAY, Color.LTGRAY);

        // Further customize renderer to be more "aesthetically pleasing"
		mChartRenderer.getSeriesRendererAt(0).setDisplayChartValues(true);  // Total Length
		mChartRenderer.getSeriesRendererAt(1).setDisplayChartValues(true);  // Noise Length
// FIXME: set axis label counts (or not... maybe that's the "view" we want...) ... might be ok...
		mChartRenderer.setXLabels(12);
		mChartRenderer.setYLabels(10);
		mChartRenderer.setXLabelsAlign(Align.LEFT);
		mChartRenderer.setYLabelsAlign(Align.LEFT);
		mChartRenderer.setPanEnabled(true, false); // FIXME: review ... X and Y pan/zoom
		// renderer.setZoomEnabled(false); // FIXME: review ... X and Y pan/zoom
		mChartRenderer.setZoomRate(1.1f);
		mChartRenderer.setBarSpacing(0.5f);
		mChartRenderer.setShowGrid(true);
//		renderer.setInScroll(true); // hope to help with repaints...

        // Create chart view
        v = ChartFactory.getBarChartView(getActivity(), mChartDataset, mChartRenderer, Type.STACKED);
        
      Log.d(LOGTAG, "DEBUG 99 makeChartView()");
      return v;
    } // END makeChartView()
	
    
    
    
	
	// ///////////////////////////////////////////////////
	// LoaderManager.LoaderCallbacks<Cursor> implementations -- FIXME: another very warty area - with a lot of code repeated from the SessionDetailFragment and SessionsListFragment...
	/**
	 * LoaderManager callback to create new loaders.
	 */
	public Loader<Cursor> onCreateLoader(int id, Bundle args) {
		Log.d(LOGTAG, "onCreateLoader() 00 id: " + id);

		String[] lproj;
		String lselc = null;
		String[] lsela = { "" };
		String lsort = null; // FIXME: set this up for SessionsDB.PRACTICE_DATE_STAMP
		Uri luri;
		
		// Setup projection		
		Log.d(LOGTAG, "onCreateLoader() 14 id: " + id);			
		lproj = new String[] { SessionsDB._ID, 
				SessionsDB.PRACTICE_DATE_STAMP, 
				SessionsDB.PRACTICE_LENGTH_TOTAL,
				SessionsDB.PRACTICE_LENGTH_NOISE};
		luri = SessionsDB.CONTENT_PRACTICES_URI;

		Log.d(LOGTAG, "onCreateLoader() 20 id: " + id);		
		// Get cursor
		CursorLoader lcursLoader = new CursorLoader(getActivity(),
				luri, lproj, lselc, lsela, lsort);
		Log.d(LOGTAG, "onCreateLoader() 99");
		return lcursLoader;
	} // END onCreateLoader()

	/**
	 * LoaderManager callback to process when loading cursor data is completed.
	 */
	public void onLoadFinished(Loader<Cursor> loader, Cursor data) {
		Log.d(LOGTAG, "onLoadFinished() 00");
		int lrows;
		
		if (null != data) {
			lrows = data.getCount();
			data.moveToFirst();
		} else {
			lrows = 0;
		}
		
		Log.d(LOGTAG, "onLoadFinished() 10 chart rows: " + lrows);

		long ldate;
		long ltotalMS;
		 int ltotalMin;
		long lnoiseMS;
		 int lnoiseMin; 
		for (int i = 0; i < lrows; i++) { // FIXME: possibly use an iterator
			ldate = data.getLong(1);
			ltotalMS = data.getLong(2);
			ltotalMin = SessionsDB.longSessionMinInt(ltotalMS);
			lnoiseMS = data.getLong(3);
			lnoiseMin = SessionsDB.longSessionMinInt(lnoiseMS);
			
			// convert data to a "common format" for the chart; for now hard-coded to minutes, but this could be hours 

			Log.d(LOGTAG, "onLoadFinished() 30    loop i:" + i + " date:" + ldate + " total: " + ltotalMS + " noise: " + lnoiseMS);

			Log.d(LOGTAG, "onLoadFinished() 34      DATE      date: " + SessionsDB.longDateString(ldate));
			Log.d(LOGTAG, "onLoadFinished() 35         tot minutes: " + SessionsDB.longSessionMinInt(ltotalMS));
			Log.d(LOGTAG, "onLoadFinished() 36         noisminutes: " + SessionsDB.longSessionMinInt(lnoiseMS));
//			mChartSeriesTotal.add(x, y)
			mChartSeriesTotal.add(i+1, ltotalMin);
			mChartSeriesNoise.add(i+1, lnoiseMin);
			data.moveToNext();
		}
		//// Lazy start -- add to the series ; possible improvement as a next step might be to delete series and re-populate.
		//mCurrentSeries.add(x, y);
		
		// update maximums
		double lmaxLen = mChartSeriesTotal.getMaxY();
		mChartRenderer.setYAxisMax(lmaxLen+5); // FIXME: should add a buffer of ~10%, instead of a fixed value

		mChartView.repaint();
		
		Log.d(LOGTAG, "onLoadFinished() 99");		
	} // END onLoadFInished()

	/**
	 * LoaderManager callback when releasing data.
	 */
	public void onLoaderReset(Loader<Cursor> loader) {
		Log.d(LOGTAG, "onLoaderReset() 00");
		// presumably nothing needs to be cleaned from the AChartEngine
//		mAdapterSessions.swapCursor(null);
	}


    
    
	
	
	///////////////////////////////////////////////////////
	//
	// Below -- Utility methods brought in from AbstractDemoChart from ACE -- FIXME: this needs review and cleanup
	//
	protected XYMultipleSeriesDataset buildDataset(String[] titles,
			List<double[]> xValues, List<double[]> yValues) {
		XYMultipleSeriesDataset dataset = new XYMultipleSeriesDataset();
		addXYSeries(dataset, titles, xValues, yValues, 0);
		return dataset;
	}

	protected void addXYSeries(XYMultipleSeriesDataset dataset, String[] titles,
			List<double[]> xValues, List<double[]> yValues, int scale) {
		int length = titles.length;
		for (int i = 0; i < length; i++) {
			XYSeries series = new XYSeries(titles[i], scale);
			double[] xV = xValues.get(i);
			double[] yV = yValues.get(i);
			int seriesLength = xV.length;
			for (int k = 0; k < seriesLength; k++) {
				series.add(xV[k], yV[k]);
			}
			dataset.addSeries(series);
		}
	}

	/**
	 * Builds an XY multiple series renderer.
	 * 
	 * @param colors
	 *            the series rendering colors
	 * @param styles
	 *            the series point styles
	 * @return the XY multiple series renderers
	 */
	protected XYMultipleSeriesRenderer buildRenderer(int[] colors,
			PointStyle[] styles) {
		XYMultipleSeriesRenderer renderer = new XYMultipleSeriesRenderer();
		setRenderer(renderer, colors, styles);
		return renderer;
	}

	protected void setRenderer(XYMultipleSeriesRenderer renderer, int[] colors,
			PointStyle[] styles) {
		renderer.setAxisTitleTextSize(16);
		renderer.setChartTitleTextSize(20);
		renderer.setLabelsTextSize(15);
		renderer.setLegendTextSize(15);
		renderer.setPointSize(5f);
		renderer.setMargins(new int[] { 20, 30, 15, 20 });
		int length = colors.length;
		for (int i = 0; i < length; i++) {
			XYSeriesRenderer r = new XYSeriesRenderer();
			r.setColor(colors[i]);
			r.setPointStyle(styles[i]);
			renderer.addSeriesRenderer(r);
		}
	}

	/**
	 * Sets a few of the series renderer settings.
	 * 
	 * @param renderer
	 *            the renderer to set the properties to
	 * @param title
	 *            the chart title
	 * @param xTitle
	 *            the title for the X axis
	 * @param yTitle
	 *            the title for the Y axis
	 * @param xMin
	 *            the minimum value on the X axis
	 * @param xMax
	 *            the maximum value on the X axis
	 * @param yMin
	 *            the minimum value on the Y axis
	 * @param yMax
	 *            the maximum value on the Y axis
	 * @param axesColor
	 *            the axes color
	 * @param labelsColor
	 *            the labels color
	 */
	protected void setChartSettings(XYMultipleSeriesRenderer renderer,
			String title, String xTitle, String yTitle, double xMin,
			double xMax, double yMin, double yMax, int axesColor,
			int labelsColor) {
		renderer.setChartTitle(title);
		renderer.setXTitle(xTitle);
		renderer.setYTitle(yTitle);
		renderer.setXAxisMin(xMin);
		renderer.setXAxisMax(xMax);
		renderer.setYAxisMin(yMin);
		renderer.setYAxisMax(yMax);
		renderer.setAxesColor(axesColor);
		renderer.setLabelsColor(labelsColor);
	}

	/**
	 * Builds an XY multiple time dataset using the provided values.
	 * 
	 * @param titles
	 *            the series titles
	 * @param xValues
	 *            the values for the X axis
	 * @param yValues
	 *            the values for the Y axis
	 * @return the XY multiple time dataset
	 */
	protected XYMultipleSeriesDataset buildDateDataset(String[] titles,
			List<Date[]> xValues, List<double[]> yValues) {
		XYMultipleSeriesDataset dataset = new XYMultipleSeriesDataset();
		int length = titles.length;
		for (int i = 0; i < length; i++) {
			TimeSeries series = new TimeSeries(titles[i]);
			Date[] xV = xValues.get(i);
			double[] yV = yValues.get(i);
			int seriesLength = xV.length;
			for (int k = 0; k < seriesLength; k++) {
				series.add(xV[k], yV[k]);
			}
			dataset.addSeries(series);
		}
		return dataset;
	}

	/**
	 * Builds a category series using the provided values.
	 * 
	 * @param titles
	 *            the series titles
	 * @param values
	 *            the values
	 * @return the category series
	 */
	protected CategorySeries buildCategoryDataset(String title, double[] values) {
		CategorySeries series = new CategorySeries(title);
		int k = 0;
		for (double value : values) {
			series.add("Project " + ++k, value);
		}

		return series;
	}

	/**
	 * Builds a multiple category series using the provided values.
	 * 
	 * @param titles
	 *            the series titles
	 * @param values
	 *            the values
	 * @return the category series
	 */
	protected MultipleCategorySeries buildMultipleCategoryDataset(String title,
			List<String[]> titles, List<double[]> values) {
		MultipleCategorySeries series = new MultipleCategorySeries(title);
		int k = 0;
		for (double[] value : values) {
			series.add(2007 + k + "", titles.get(k), value);
			k++;
		}
		return series;
	}

	/**
	 * Builds a category renderer to use the provided colors.
	 * 
	 * @param colors
	 *            the colors
	 * @return the category renderer
	 */
	protected DefaultRenderer buildCategoryRenderer(int[] colors) {
		DefaultRenderer renderer = new DefaultRenderer();
		renderer.setLabelsTextSize(15);
		renderer.setLegendTextSize(15);
		renderer.setMargins(new int[] { 20, 30, 15, 0 });
		for (int color : colors) {
			SimpleSeriesRenderer r = new SimpleSeriesRenderer();
			r.setColor(color);
			renderer.addSeriesRenderer(r);
		}
		return renderer;
	}

	/**
	 * Builds a bar multiple series dataset using the provided values.
	 * 
	 * @param titles
	 *            the series titles
	 * @param values
	 *            the values
	 * @return the XY multiple bar dataset
	 */
	protected XYMultipleSeriesDataset buildBarDataset(String[] titles,
			List<double[]> values) {
		XYMultipleSeriesDataset dataset = new XYMultipleSeriesDataset();
		int length = titles.length;
		for (int i = 0; i < length; i++) {
			CategorySeries series = new CategorySeries(titles[i]);
			double[] v = values.get(i);
			int seriesLength = v.length;
			for (int k = 0; k < seriesLength; k++) {
				series.add(v[k]);
			}
			dataset.addSeries(series.toXYSeries());
		}
		return dataset;
	}

	/**
	 * Builds a bar multiple series renderer to use the provided colors.
	 * 
	 * @param colors
	 *            the series renderers colors
	 * @return the bar multiple series renderer
	 */
	protected XYMultipleSeriesRenderer buildBarRenderer(int[] colors) {
		XYMultipleSeriesRenderer renderer = new XYMultipleSeriesRenderer();
		renderer.setAxisTitleTextSize(16);
		renderer.setChartTitleTextSize(20);
		renderer.setLabelsTextSize(15);
		renderer.setLegendTextSize(15);
		int length = colors.length;
		for (int i = 0; i < length; i++) {
			SimpleSeriesRenderer r = new SimpleSeriesRenderer();
			r.setColor(colors[i]);
			renderer.addSeriesRenderer(r);
		}
		return renderer;
	}


} // END class SessionsSummaryFragment
